// Copyright (c) 2007 Keith D Gregory
package com.kdgregory.example.ref.queue;

import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.SoftReference;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;


/**
 *  Creates lots of buffers, held through reference objects, to demonstrate how
 *  those objects will be added to a queue when they're cleared.
 */
public class QueueDemo
{
    public static void main(String[] argv) throws Exception
    {
        // must hold strong references to the refobjs themselves, or they'll
        // simply disappear ... queue is NOT sufficient
        List<SoftReference<byte[]>> refs = new ArrayList<SoftReference<byte[]>>();
        ReferenceQueue<byte[]> queue = new ReferenceQueue<byte[]>();
        
        for (int ii = 0 ; ii < 10000 ; ii++)
        {
            SoftReference<byte[]> ref = new SoftReference<byte[]>(new byte[10000], queue);
            System.err.println(ii + ": created " + ref);
            refs.add(ref);
            
            Reference<? extends byte[]> r2;
            while ((r2 = queue.poll()) != null)
            {
                System.err.println("cleared " + r2);
            }
        }
    }
}
