// Copyright (c) 2007 Keith D Gregory
package com.kdgregory.example.ref.weak;

import java.util.HashMap;
import java.util.Map;

/**
 *  An example of a canonicalizing map for strings, which never releases the
 *  items that are added to it. Contrast with {@link WeakCanonicalingMap}.
 *  <p>
 *  This class is not thread-safe. To make it thread-safe, you could either
 *  synchronize all methods, or use Collections.synchronizedMap() to wrap
 *  the map, or use SynchronizedHashMap.
 */
public class SimpleCanonicalizingMap
{
    private Map<String,String> _map = new HashMap<String,String>();
    
    /**
     *  Returns a canonical instance of the passed string: the first string
     *  interned by this map (which may be the passed string).
     */
    public synchronized String intern(String str)
    {
        if (_map.containsKey(str))
            return _map.get(str);
        _map.put(str, str);
        return str;
    }
    
    
    /**
     *  Returns the number of entries in the map. This is useful for a demo,
     *  probably not so much in real life (except as a debugging tool).
     */
    public synchronized int size()
    {
        return _map.size();
    }
}
